const vscode = require('vscode');
const { exec } = require('child_process');
const path = require('path');
const fs = require('fs');

function activate(context) {
    console.log('Git Actions extension is now active!');
    let revert = vscode.commands.registerCommand('git-actions.revert', (uri) => {
        executeGitCommand('Revert', getRelativePath(uri));
    });

    let stage = vscode.commands.registerCommand('git-actions.stage', (uri) => {
        executeGitCommand('Stage', getRelativePath(uri));
    });

    let unstage = vscode.commands.registerCommand('git-actions.unstage', (uri) => {
        executeGitCommand('Unstage', getRelativePath(uri));
    });

    let stageAndCommit = vscode.commands.registerCommand('git-actions.stageAndCommit', (uri) => {
        executeGitCommand('Stage and commit', getRelativePath(uri));
    });

    let deleteFile = vscode.commands.registerCommand('git-actions.deleteFile', (uri) => {
        executeGitCommand('Delete File', getRelativePath(uri), uri.fsPath);
    });

    context.subscriptions.push(revert, stage, unstage, stageAndCommit, deleteFile);
}

function getRelativePath(uri) {
    if (vscode.workspace.workspaceFolders) {
        const workspaceFolder = vscode.workspace.workspaceFolders[0];
        return path.relative(workspaceFolder.uri.fsPath, uri.fsPath);
    }
    return uri.fsPath; // Fallback to full path if no workspace folder
}

function executeGitCommand(action, relativePath, fullPath = null) {
    let command = '';
    switch (action) {
        case 'Revert':
            command = `git checkout -- "${relativePath}"`;
            break;
        case 'Stage':
            command = `git add "${relativePath}"`;
            break;
        case 'Unstage':
            command = `git reset HEAD "${relativePath}"`;
            break;
        case 'Stage and commit':
            vscode.window.showInputBox({ prompt: 'Enter commit message' })
                .then(message => {
                    if (message) {
                        command = `git add "${relativePath}" && git commit -m "${message}"`;
                        runCommand(command, action, relativePath);
                    } else {
                        vscode.window.showWarningMessage('Commit cancelled: No message provided');
                    }
                });
            return;
        case 'Delete File':
            fs.stat(fullPath, (err, stats) => {
                if (err) {
                    vscode.window.showErrorMessage(`Error accessing path: ${err.message}`);
                    return;
                }
                const isDirectory = stats.isDirectory();
                const itemType = isDirectory ? 'directory' : 'file';
                const warningMessage = `WARNING: This action will permanently delete the ${itemType} "${relativePath}" and all its contents from both the file system and Git. This action cannot be undone. Are you sure you want to proceed?`;
                vscode.window.showWarningMessage(warningMessage, { modal: true }, 'Yes, Delete')
                    .then(answer => {
                        if (answer === 'Yes, Delete') {
                            deleteAndRemoveFromGit(fullPath, relativePath, isDirectory);
                        } else {
                            vscode.window.showInformationMessage(`${itemType.charAt(0).toUpperCase() + itemType.slice(1)} deletion cancelled.`);
                        }
                    });
            });
            return;
    }

    if (command) {
        runCommand(command, action, relativePath);
    }
}


function deleteAndRemoveFromGit(fullPath, relativePath, isDirectory) {
    const workspaceRoot = vscode.workspace.workspaceFolders ? vscode.workspace.workspaceFolders[0].uri.fsPath : '';
    
    const deleteFunction = isDirectory ? (callback) => rimraf(fullPath, callback) : fs.unlink;
    
    deleteFunction(fullPath, (err) => {
        if (err) {
            vscode.window.showErrorMessage(`Error deleting ${isDirectory ? 'directory' : 'file'} from file system: ${err.message}`);
            console.error(`Error deleting ${isDirectory ? 'directory' : 'file'} from file system:`, err);
            return;
        }
        
        exec(`git rm -rf "${relativePath}"`, { cwd: workspaceRoot }, (error, stdout, stderr) => {
            if (error) {
                vscode.window.showErrorMessage(`Error removing ${isDirectory ? 'directory' : 'file'} from Git: ${error.message}`);
                console.error(`Error removing ${isDirectory ? 'directory' : 'file'} from Git:`, error);
                return;
            }
            if (stderr) {
                vscode.window.showErrorMessage(`Error removing ${isDirectory ? 'directory' : 'file'} from Git: ${stderr}`);
                console.error(`Error removing ${isDirectory ? 'directory' : 'file'} from Git:`, stderr);
                return;
            }
            vscode.window.showInformationMessage(`${isDirectory ? 'Directory' : 'File'} ${relativePath} has been permanently deleted.`);
        });
    });
}

function runCommand(command, action, relativePath) {
    setTimeout(() => {
        vscode.window.showInformationMessage(`Executing ${action} on ${relativePath}...`);
    }, 100);

    const workspaceRoot = vscode.workspace.workspaceFolders ? vscode.workspace.workspaceFolders[0].uri.fsPath : '';
    
    exec(command, { cwd: workspaceRoot }, (error, stdout, stderr) => {
        if (error) {
            vscode.window.showErrorMessage(`Error executing ${action}: ${error.message}`);
            console.error(`Error executing ${action}:`, error);
            return;
        }
        if (stderr) {
            vscode.window.showErrorMessage(`Error executing ${action}: ${stderr}`);
            console.error(`Error executing ${action}:`, stderr);
            return;
        }
        vscode.window.showInformationMessage(`${action} executed successfully on ${relativePath}`);
        console.log(`${action} executed successfully:`, stdout);
    });
}

function deactivate() {}

module.exports = {
    activate,
    deactivate
};